@@ -1,10 +1,12 @@
 use utf8;
 use strict;
 
-use XML::Pastor::Builtin::SimpleType;
 
 #======================================================================
 package XML::Pastor::Builtin::Union;
+
+use XML::Pastor::Builtin::SimpleType;
+
 our @ISA = qw(XML::Pastor::Builtin::SimpleType);
 
 #--------------------------------------------------------------
@@ -12,7 +14,7 @@ sub xml_validate {
 	my $self	= shift;
 	my $path	= shift || '';	
 	my $type	= $self->XmlSchemaType();
-	my $value	= $self->value;
+	my $value	= $self->__value;
 	my $members	= $type->memberClasses || [];
 	
 	unless (@$members) {
@@ -21,7 +23,7 @@ sub xml_validate {
 	
 	foreach my $class (@$members) {
 		if (UNIVERSAL::can($class,"xml_validate")) {		
-			my $object = $class->new(value => $value);
+			my $object = $class->new(__value => $value);
 			if ($object->xml_validate(@_)) {
 				return 1;
 			}
