@@ -1,13 +1,14 @@
 use utf8;
 use strict;
 
+#=======================================================
+package XML::Pastor::Generator;
+
 use Data::Dumper;
 use IO::File;
 use File::Path;
 use File::Spec;
 use Class::Accessor;
-
-package XML::Pastor::Generator;
 use XML::Pastor::Util qw(mergeHash);
 
 our @ISA = qw(Class::Accessor);
@@ -71,7 +72,8 @@ sub _generateSingle {
 	my $args	= {@_};
 	my $model	= $args->{model};
 	my $mode	= $args->{mode};
-	my $module	= $args->{module};	
+	my $module	= $args->{module};
+	my $class_prefix = $args->{class_prefix};	
 	my $code	= $self->_fabricatePrelude(@_);	
 	my $destination		= $args->{destination} || '/tmp/lib/perl/';
 	my $verbose	= $self->{verbose} || 0;
@@ -96,9 +98,11 @@ sub _generateSingle {
 		return $code;
 	}else { 
 		# generate module
-		$destination .= '/' unless ($destination =~ /\/$/);		
+		$destination .= '/' if ($destination && ($destination !~ /\/$/));
+		$class_prefix .= '::' if ($class_prefix && ($class_prefix !~ /::$/));		
 		my $file	= $module or  die "Pastor:Generator:_generateSingle:A 'module' name is required!\n"; 
-		$file 		= $destination . $file . '.pm';	
+		
+		$file 		= $destination .  $file . '.pm';	
 		$file 		=~ s/::/\//g;		
 		$self->_writeCode(@_, file=>$file, code=>$code);	
 	}
@@ -114,12 +118,14 @@ sub _generateMultiple {
 	my $model	= $args->{model};
 	my $types	= $model->type();
 	my $destination		= $args->{destination} || '/tmp/lib/perl/';
-	$destination .= '/' unless ($destination =~ /\/$/);
+	$destination .= '/' if ($destination && ($destination !~ /\/$/));
 	
 	foreach my $items ($model->type(), 	$model->element()) {
 		foreach my $name (sort keys %$items) {
 			my $object 	= $items->{$name};
 			my $code	= $self->_fabricateCode(@_, object=>$object);
+			$code 	   	.= "\n\n__END__\n\n";
+			$code		.=$self->_fabricatePod(@_, object=>$object);
 		
 			my $file	= $object->class();
 			$file = $destination . $file . '.pm';
@@ -141,7 +147,9 @@ sub _fabricatePrelude {
 	$code .= "\n#PASTOR: Code generated by XML::Pastor/" . $XML::Pastor::VERSION . " at '" . localtime() . "'\n";
 	$code .= "\nuse utf8;";
 	$code .= "\nuse strict;";	
-#	$code .= "\nuse warnings;";	
+#	$code .= "\nuse warnings;";
+#	$code .= "\nno warnings qw(uninitialized);";
+
 	$code .= "\n\nuse XML::Pastor;\n\n";	
 	return $code;	
 }
@@ -253,8 +261,9 @@ sub _calculateUses {
 		# Consider the class as 'used' unless it starts with XML::Pastor which is handled differently
 		# But XML::Pastor::Test::* are used for testing purposes. 
 		SWITCH: {
-			$class =~ /^XML::Pastor::Test/			and do {$result->{$class} = 1; last SWITCH;};
-			$class =~ /^XML::Pastor::/				and do {last SWITCH;};			
+			(($class =~ /^XML::Pastor::/) && 
+			($class !~ /^XML::Pastor::Test/))				and do {last SWITCH;};
+			OTHERWISE:										{ $result->{$class}=1; last SWITCH;}			
 		}		
 		
 	}
@@ -265,8 +274,10 @@ sub _calculateUses {
 		# Consider the class as 'used' unless it starts with XML::Pastor which is handled differently
 		# But XML::Pastor::Test::* are used for testing purposes. 
 		SWITCH: {
-			$class =~ /^XML::Pastor::Test/			and do {$result->{$class} = 1; last SWITCH;};
-			$class =~ /^XML::Pastor::/				and do {last SWITCH;};			
+			(($class =~ /^XML::Pastor::/) && 
+			($class !~ /^XML::Pastor::Test/))				and do {last SWITCH;};
+			OTHERWISE:										{ $result->{$class}=1; last SWITCH;}			
+						
 		}		
 		
 	}
@@ -277,20 +288,23 @@ sub _calculateUses {
 			# Consider the class as 'used' unless it starts with XML::Pastor which is handled differently
 			# But XML::Pastor::Test::* are used for testing purposes. 
 			SWITCH: {
-				$class =~ /^XML::Pastor::Test/			and do {$result->{$class} = 1; last SWITCH;};
-				$class =~ /^XML::Pastor::/				and do {last SWITCH;};			
+				(($class =~ /^XML::Pastor::/) && 
+				($class !~ /^XML::Pastor::Test/))				and do {last SWITCH;};
+				OTHERWISE:										{ $result->{$class}=1; last SWITCH;}							
 			}		
 		}
 	}
 
-	if (UNIVERSAL::can($object, "memberClasses") && $object->baseClasses()) {
+	if (UNIVERSAL::can($object, "memberClasses") && $object->memberClasses()) {
 		my $mbcs = $object->memberClasses();
 		foreach my $class (@$mbcs) {		
 			# Consider the class as 'used' unless it starts with XML::Pastor which is handled differently
 			# But XML::Pastor::Test::* are used for testing purposes. 
 			SWITCH: {
-				$class =~ /^XML::Pastor::Test/			and do {$result->{$class} = 1; last SWITCH;};
-				$class =~ /^XML::Pastor::/				and do {last SWITCH;};			
+				(($class =~ /^XML::Pastor::/) && 
+				($class !~ /^XML::Pastor::Test/))				and do {last SWITCH;};
+				OTHERWISE:										{ $result->{$class}=1; last SWITCH;}			
+				
 			}		
 		}
 	}
@@ -315,6 +329,114 @@ sub _calculateUses {
 	return $result;
 }
 
+#--------------------------------------------
+# Fabricate the POD for one given class (type).
+#--------------------------------------------
+sub _fabricatePod {
+	my $self 	= shift;
+	my $args	= {@_};
+	my $object	= $args->{object} or die "Pastor: _fabricatePod: Need a type!\n";
+	my $style	= $args->{style};	
+	my $class	= $object->class();		
+	my $isa		= $object->baseClasses()  || [];	
+	my $verbose	= $self->{verbose} || 0;
+	my $pod	= "";
+	
+	print STDERR "\nFabricating POD for class '$class' ..." if ($verbose >= 3);
+
+	# NAME	
+	$pod .= "\n\n=head1 NAME\n\nB<$class>  -  A class generated by L<XML::Pastor> . \n";
+
+	# DESCRIPTION
+	if (defined $object->documentation) {
+		$pod .= "\n\n=head1 DESCRIPTION\n";
+		foreach my $doc (@{$object->documentation}) {
+			my $text = $doc->text;
+			$text =~ s/^\s+//;	# chop leading space.
+			$pod .= "\n" . $text . "\n";
+		}
+	}
+	
+	
+	# ISA
+	$pod .= "\n\n=head1 ISA\n\nThis class descends from " . join(', ', map {"L<$_>"} @$isa) . ".\n";
+	
+	# CODE GENERATION
+	$pod .= "\n\n=head1 CODE GENERATION\n\nThis module was automatically generated by L<XML::Pastor> version " . $XML::Pastor::VERSION . " at '" . localtime() . "'\n";
+	
+	# ATTRIBUTE Accessors			
+	if (UNIVERSAL::can($object, "attributes") && scalar(@{$object->attributes})) {
+		my $attributes = $object->attributes();
+		my $attribPfx = "";
+		$attribPfx = $object->attributePrefix() if (UNIVERSAL::can($object, "attributePrefix"));
+		my $attribInfo = $object->effectiveAttributeInfo();
+		my $elementInfo = UNIVERSAL::can($object, 'effectiveElementInfo') ? $object->effectiveElementInfo() : {};
+		
+		$pod .= "\n\n=head1 ATTRIBUTE ACCESSORS\n";
+		$pod .= "\n=over\n";
+		
+		foreach my $attribute_name (sort @$attributes) {
+			my $accessor    = $attribPfx . $attribute_name;
+			my $attrib  	= $attribInfo->{$attribute_name};
+			next unless (defined $attrib);
+			
+			$pod .= "\n=item B<$accessor>()";
+			
+			# Attribute accessor alias (if there is no conflict with an element name)
+			unless (exists($elementInfo->{$attribute_name})) {
+				$pod .= ", B<$attribute_name>()";
+			}	 
+			
+			my $aclass		= $attrib->class;
+			if (defined $aclass) {			
+				$pod .= "      - See L<$aclass>.";
+			}
+			
+			$pod .= "\n";
+		}
+		$pod .= "\n=back\n";				
+	}
+	
+	
+	# CHILD ELEMENT accessors
+	if (UNIVERSAL::can($object, "elements") && scalar(@{$object->elements})) {
+		my $elementInfo = $object->effectiveElementInfo();
+
+		$pod .= "\n\n=head1 CHILD ELEMENT ACCESSORS\n";
+		$pod .= "\n=over\n";
+				
+		foreach my $accessor (sort @{$object->elements()}) {
+			my $element = $elementInfo->{$accessor};
+			next unless (defined $element);
+
+			$pod .= "\n=item B<$accessor>()";
+			my $aclass		= $element->class;
+			if (defined $aclass) {			
+				$pod .= "      - See L<$aclass>.";
+			}
+			
+			$pod .= "\n";
+			
+		}
+		$pod .= "\n=back\n";	
+	}
+	
+	# SEE ALSO		
+	my @see_also = (@$isa, 'XML::Pastor', 'XML::Pastor::Type', 'XML::Pastor::ComplexType', 'XML::Pastor::SimpleType');
+	@see_also  = map {"L<$_>"} @see_also;
+	
+	$pod .= "\n\n=head1 SEE ALSO\n\n";
+	$pod .= join (", ", @see_also);
+	$pod .= "\n";
+	
+	# EPILOGUE		
+	$pod .= "\n\n=cut\n";
+	
+	 		
+	return $pod;
+}	
+
+
 #--------------------------------------------
 # Dump the data for a given "schema type". 
 # Note the use of "Deepcopy"  and "Terse" 
